@@ -21,6 +21,7 @@
  * ctime option added, check ctime in addition to atime, for those cases where
  * files get created with an old date in atime and mtime (e.g. via Samba from
  * a PC).
+ * See changelog for later changes.
  */
 
 #define _LARGEFILE_SOURCE 1     /* handle >2GB files where possible */
@@ -172,6 +173,7 @@ protect_entry * protect_table;	/* Global */
 
 int	logLevel = LOG_NORMAL;
 int     delayMax = 0;
+int     runtimeMax = 55;
 
 void
 message (const int    level,
@@ -187,6 +189,7 @@ message (const int    level,
 	switch (level) {
 	case LOG_DEBUG:
 	    where = stdout;
+            /*FALLTHRU*/
 	case LOG_REALDEBUG:
 	    fprintf (where, "debug: ");
 	    break;
@@ -465,6 +468,21 @@ cleanupDirectory (const char	   * dirname ,
 		}
 
 		if (S_ISDIR (sb.st_mode)) {
+                    char *dst;
+
+                    if ((dst = malloc(strlen(ent->d_name) + 3)) == NULL)
+                        message (LOG_FATAL, "malloc failed.\n");
+                    strcpy(dst, ent->d_name);
+                    strcat(dst, "/X");
+                    rename(ent->d_name, dst);
+                    if (errno == EXDEV) {
+                        free(dst);
+                        message (LOG_VERBOSE,
+                                 "File on different device skipped: `%s/%s'\n",
+                                 dirname, ent->d_name);
+                        continue;
+                    }
+                    free(dst);
 		    cleanupDirectory (ent->d_name, killTime, flags);
 		    message (LOG_VERBOSE,
 			     "(PID %u) Back from recursing down `%s'.\n",
@@ -507,13 +525,15 @@ cleanupDirectory (const char	   * dirname ,
 		}
 
 		if (S_ISDIR (sb.st_mode)) {
-		    if (FLAGS_TEST_P (flags)) {
-			message (LOG_VERBOSE,
-				 "Pretending to maybe remove possibly empty directory `%s'.\n",
-				 ent->d_name);
-		    }
-		    else {
-			if (dir_empty_p (ent->d_name)) {
+                    if (dir_empty_p (ent->d_name)) {
+                        if (FLAGS_TEST_P (flags)) {
+                            message (LOG_VERBOSE,
+                                     "Pretending to remove empty directory `%s'.\n",
+                                     ent->d_name);
+                            if (FLAGS_SHOWDEL_P(flags))
+                                message(LOG_NORMAL, "rmdir %s/%s\n", dirname, ent->d_name);
+                        }
+                        else {
 			    message (LOG_VERBOSE,
 				     "Removing directory `%s'.\n",
 				     ent->d_name);
@@ -535,6 +555,8 @@ cleanupDirectory (const char	   * dirname ,
 			message (LOG_VERBOSE,
 				 "Pretending to remove file `%s/%s'.\n",
 				 dirname, ent->d_name);
+                        if (FLAGS_SHOWDEL_P(flags))
+                            message(LOG_NORMAL, "rm %s/%s\n", dirname, ent->d_name);
 		    }
 		    else {
 			message (LOG_VERBOSE,
@@ -627,6 +649,7 @@ printCopyright (void)
 	     "tmpreaper -- Version: " VERSION "\n"
 #endif
 	     "(c) 1997 Karl M. Hegbloom <tmpreaper@packages.debian.org>\n"
+	     "(c) 2006 Paul Slootman    <tmpreaper@packages.debian.org>\n"
 	     "This may be freely redistributed under the terms of the GNU Public License.\n");
 }
 /*	               1         2         3         4         5         6         7         8
@@ -638,8 +661,9 @@ usage(void)
 {
     printCopyright ();
     fprintf (stderr, "\n"
-	     "tmpreaper [-htvfmMsa] [--help] [--test] [--verbose] [--force]\n"
-	     "          [--mtime] [--mtime-dir] [--symlinks] [--all]\n"
+	     "tmpreaper [-htvfmMsadVT] [--help] [--test] [--verbose] [--force]\n"
+	     "          [--delay=99] [--runtime=99] [--showdeleted]\n"
+	     "          [--ctime] [--mtime] [--mtime-dir] [--symlinks] [--all]\n"
 	     "          [[--protect '<shell_pattern>']...] <time> <dirs>...\n"
 	     "<time> is time since a file in a <dir> was last accessed.\n"
 	     "It defaults to hours, or you may suffix with `s', `m', `h', or `d'.\n\n");
@@ -686,6 +710,7 @@ main (int     argc,
 	{ "protect",	required_argument,	NULL, 'p' },
 	{ "delay",	optional_argument,	NULL, 'd' },
 	{ "showdeleted",no_argument,	        NULL, 'V' },
+	{ "runtime",	required_argument,	NULL, 'T' },
 	{ 0, 0, 0, 0 }
     };
 
@@ -695,7 +720,7 @@ main (int     argc,
     while (1) {
 	long_index = 0;
 
-	arg = getopt_long(argc, argv, "htvfcmMsa", options, &long_index);
+	arg = getopt_long(argc, argv, "htv::fcmMsap:d::VT:", options, &long_index);
 	if (arg == -1)
 	    break;
 
@@ -726,10 +751,18 @@ main (int     argc,
 	    break;
 
 	case 'c':
+            if (flags & FLAGS_MTIME == FLAGS_MTIME) {
+                message(LOG_FATAL, "--ctime conflicts with --mtime option\n");
+                /*NOTREACHED*/
+            }
 	    flags |= FLAGS_CTIME;
 	    break;
 
 	case 'm':
+            if (flags & FLAGS_CTIME == FLAGS_CTIME) {
+                message(LOG_FATAL, "--mtime conflicts with --ctime option\n");
+                /*NOTREACHED*/
+            }
 	    flags |= FLAGS_MTIME;
 	    break;
 
@@ -762,6 +795,14 @@ main (int     argc,
         case 'V':
             flags |= FLAGS_SHOWDEL;
             break;
+
+        case 'T':
+            runtimeMax = atoi(optarg);
+            message(LOG_DEBUG, "runtime is %d seconds max\n", runtimeMax);
+            if (runtimeMax < 5) {
+                message(LOG_FATAL, "runtime must be at least 5 seconds\n");
+                /*NOTREACHED*/
+            }
 	}
     }
 
@@ -881,7 +922,7 @@ main (int     argc,
     }
       
     signal(SIGALRM, sig_alarmhandler);
-    alarm(55);
+    alarm(runtimeMax);
 
     /* For each directory in the cleanup list, cd there, glob, then delete */
     for (adir = optind; adir < argc; adir++) {
